from typing import List, Set

from id_broker_common.pseudonyms import generate_pseudonym
from id_broker_common.pseudonyms_udm_rest import (
    get_service_provider_secret,
    get_settings_data_content,
    ldap_settings,
)
from ucsschool.kelvin.client import KelvinObject


def get_roles_for_school(roles: List[str], school: str) -> Set[str]:
    """
    Takes a list of ucsschool_roles and returns a list of user roles for the given school.

    >>> get_roles_for_school(["teacher:school:School1", "student:school:School2"], "School1")
    {'teacher'}

    :param roles: The list of ucsschool_roles to filter
    :param school: The school to filter the roles for
    :return: The list of user roles for the given school
    :raises ValueError: If any of the role strings is malformed
    """
    filtered_roles = set()
    for role in roles:
        role_parts = role.split(":")
        if len(role_parts) != 3 or not all(role_parts):
            raise ValueError(f"The role {role} is malformed!")
        if (
            role_parts[1] == "school"
            and role_parts[2] == school
            and role_parts[0] in ("student", "teacher", "staff")
        ):
            filtered_roles.add(role_parts[0])
    return filtered_roles


async def get_pseudonyms_for_providers(entry_uuid: str, school_authority: str) -> KelvinObject:
    """
    Generates pseudonyms for all service providers, which are registered
    and sets the corresponding UDM property.
    """
    ldap_base = ldap_settings().ldap_base
    dn = f"cn=IDBrokerServiceProviderMappings,cn=IDBrokerSettings,cn=univention,{ldap_base}"
    provider_to_udm_prop = await get_settings_data_content(dn)
    for sp, udm_prop in provider_to_udm_prop.items():
        # todo could be cached for optimization
        service_provider_salt = await get_service_provider_secret(sp)
        yield udm_prop, generate_pseudonym(service_provider_salt, entry_uuid, school_authority)
